summaryrefslogtreecommitdiff
path: root/libavformat/rtmpcrypt.c
diff options
context:
space:
mode:
authorSamuel Pitoiset <samuel.pitoiset@gmail.com>2012-07-20 16:36:47 +0200
committerMartin Storsjö <martin@martin.st>2012-07-23 16:32:09 +0300
commit08cd95e8a37674401ed24e5e6f4f7402edb7fdeb (patch)
tree3b0c2a52c4f70859b9a72ecab74f781d4a092f65 /libavformat/rtmpcrypt.c
parentacd554c103459ae7a5d89caa611e91bbc9bf695f (diff)
RTMPTE protocol support
Signed-off-by: Martin Storsjö <martin@martin.st>
Diffstat (limited to 'libavformat/rtmpcrypt.c')
-rw-r--r--libavformat/rtmpcrypt.c33
1 files changed, 29 insertions, 4 deletions
diff --git a/libavformat/rtmpcrypt.c b/libavformat/rtmpcrypt.c
index e5afcc94ff..6a4332de86 100644
--- a/libavformat/rtmpcrypt.c
+++ b/libavformat/rtmpcrypt.c
@@ -26,6 +26,7 @@
#include "libavutil/blowfish.h"
#include "libavutil/intreadwrite.h"
+#include "libavutil/opt.h"
#include "libavutil/rc4.h"
#include "libavutil/xtea.h"
@@ -37,11 +38,13 @@
/* protocol handler context */
typedef struct RTMPEContext {
+ const AVClass *class;
URLContext *stream; ///< TCP stream
FF_DH *dh; ///< Diffie-Hellman context
struct AVRC4 key_in; ///< RC4 key used for decrypt data
struct AVRC4 key_out; ///< RC4 key used for encrypt data
int handshaked; ///< flag indicating when the handshake is performed
+ int tunneling; ///< use a HTTP connection (RTMPTE)
} RTMPEContext;
static const uint8_t rtmpe8_keys[16][16] = {
@@ -248,11 +251,17 @@ static int rtmpe_open(URLContext *h, const char *uri, int flags)
av_url_split(NULL, 0, NULL, 0, host, sizeof(host), &port, NULL, 0, uri);
- if (port < 0)
- port = 1935;
+ if (rt->tunneling) {
+ if (port < 0)
+ port = 80;
+ ff_url_join(url, sizeof(url), "ffrtmphttp", NULL, host, port, NULL);
+ } else {
+ if (port < 0)
+ port = 1935;
+ ff_url_join(url, sizeof(url), "tcp", NULL, host, port, NULL);
+ }
- /* open the tcp connection */
- ff_url_join(url, sizeof(url), "tcp", NULL, host, port, NULL);
+ /* open the tcp or ffrtmphttp connection */
if ((ret = ffurl_open(&rt->stream, url, AVIO_FLAG_READ_WRITE,
&h->interrupt_callback, NULL)) < 0) {
rtmpe_close(h);
@@ -298,6 +307,21 @@ static int rtmpe_write(URLContext *h, const uint8_t *buf, int size)
return size;
}
+#define OFFSET(x) offsetof(RTMPEContext, x)
+#define DEC AV_OPT_FLAG_DECODING_PARAM
+
+static const AVOption ffrtmpcrypt_options[] = {
+ {"ffrtmpcrypt_tunneling", "Use a HTTP tunneling connection (RTMPTE).", OFFSET(tunneling), AV_OPT_TYPE_INT, {0}, 0, 1, DEC},
+ { NULL },
+};
+
+static const AVClass ffrtmpcrypt_class = {
+ .class_name = "ffrtmpcrypt",
+ .item_name = av_default_item_name,
+ .option = ffrtmpcrypt_options,
+ .version = LIBAVUTIL_VERSION_INT,
+};
+
URLProtocol ff_ffrtmpcrypt_protocol = {
.name = "ffrtmpcrypt",
.url_open = rtmpe_open,
@@ -306,4 +330,5 @@ URLProtocol ff_ffrtmpcrypt_protocol = {
.url_close = rtmpe_close,
.priv_data_size = sizeof(RTMPEContext),
.flags = URL_PROTOCOL_FLAG_NETWORK,
+ .priv_data_class = &ffrtmpcrypt_class,
};